from django.conf.urls import patterns

# Uncomment the next two lines to enable the admin:
# from django.contrib import admin
# admin.autodiscover()

urlpatterns = patterns('staticblog.views',
    (r'^$', 'archive'),
    (r'^([\-\w]+)$', 'render_post'),
)



from django.conf import settings
import os

STATICBLOG_ROOT = os.path.abspath(os.path.dirname(__name__))

STATICBLOG_POST_DIRECTORY = getattr(
    settings, 
    'STATICBLOG_POST_DIRECTORY', 
    STATICBLOG_ROOT + '/posts/'
)

STATICBLOG_COMPILE_DIRECTORY = getattr(
    settings, 
    'STATICBLOG_COMPILE_DIRECTORY', 
    settings.MEDIA_ROOT + 'posts/'
)

STATICBLOG_STORAGE = getattr(
    settings, 
    'STATICBLOG_STORAGE', 
    'django.core.files.storage.FileSystemStorage'
)



# Django imports
from django.template import RequestContext
from django.shortcuts import render_to_response
from django.core.files.storage import get_storage_class
from django.core.files.base import ContentFile

def render_response(req, *args, **kwargs):
    """Shortcut to wrap request in RequestContext"""
    kwargs['context_instance'] = RequestContext(req)
    return render_to_response(*args, **kwargs)

# stdlib
import os
import sys
import urllib2
import hashlib

# 3rd party
import markdown
from markdown.inlinepatterns import ImagePattern, IMAGE_LINK_RE

# app
from staticblog.app_settings import STATICBLOG_COMPILE_DIRECTORY, \
                                    STATICBLOG_POST_DIRECTORY, \
                                    STATICBLOG_STORAGE

class S3ImagePattern(ImagePattern):
    """ Wrapper class to handle image matches in markdown document """

    def handleMatch(self, match):
        node = ImagePattern.handleMatch(self, match)
        # check 'src' to ensure it is local
        src = node.attrib.get('src')
        storage_class = get_storage_class(STATICBLOG_STORAGE)
        storage = storage_class()
    
        # otherwise we need to do some downloading!
        if 'http://' in src or 'https://' in src:
            img_data = urllib2.urlopen(src).read()
            md5 = hashlib.md5()
            md5.update(img_data)
            name = md5.hexdigest() + '/' + os.path.basename(src)
        else:
            with open(STATICBLOG_POST_DIRECTORY + src, 'r') as fhandle:
                img_data = fhandle.read()
            name = src
        
        print >> sys.stderr, 'Uploading ' + src
        try:
            storage.save(name, ContentFile(img_data))
            node.attrib['src'] = storage.url(name)
            print >> sys.stderr, 'Uploaded ' + src + ' to ' + storage.url(name)
        except Exception as e:
            print >> sys.stderr, str(e)
            print >> sys.stderr, '\033[01;31mUpload of %s failed\033[0m' % src
        return node




def render_post(request, post_name):
    """ Render a blog post based on a .post template

    The used template is rendered as html in the folder defined
    by STATICBLOG_COMPILE_DIRECTORY
    """

    content = ""
    mdown = markdown.Markdown(extensions = ['meta',])
    mdown.inlinePatterns['image_link'] = S3ImagePattern(IMAGE_LINK_RE, mdown)
    try:
        post_filename = STATICBLOG_POST_DIRECTORY + post_name + '.post'
        with open(post_filename, 'r') as fhandle:
            content = fhandle.read()
            html = mdown.convert(content.decode('UTF-8'))  
    except IOError as e:
        print str(e)
        with open(STATICBLOG_POST_DIRECTORY + post_name + '.preview', 'r') as f:
            content = f.read()
            html = mdown.convert(content)

    post = {
        'content' : html ,
    }

    try:
        post['date'] = mdown.Meta['date'][0]
        post['title'] = mdown.Meta['title'][0]
        post['author'] = mdown.Meta['author'][0]
        post['summary'] = mdown.Meta['summary'][0]
    except:
        pass

    meta = {}
    if 'title' in post: 
        meta['title'] = post['title']

    return render_response(
        request, 
        'staticblog/post.html', 
        {'post' : post, 'meta' : meta}
    )

def archive(request):
    mdown = markdown.Markdown(extensions = ['meta',])
    posts = []
    import string
    for item in os.listdir(STATICBLOG_COMPILE_DIRECTORY):       
        if item.endswith('.post'):
            continue
        try:
            with open(STATICBLOG_POST_DIRECTORY + item + '.post') as fhandle:
                content = fhandle.read()
                mdown.convert(content.decode('UTF-8'))  
                post = {
                    'name' : item,
                }
                if 'title' in mdown.Meta and len(mdown.Meta['title'][0]) > 0:
                    post['title'] = mdown.Meta['title'][0]
                else:
                    post['title'] = string.capwords(item.replace('-', ' '))
                if 'date' in mdown.Meta:
                    post['date'] = mdown.Meta['date'][0]

                posts.append(post)
        except:
            pass

    from operator import itemgetter
    posts = sorted(posts, key=itemgetter('date'))     
    posts.reverse()
    return render_response(
        request, 
        'staticblog/archive.html', 
        {'posts' : posts}
    )
            



from django.core.management.base import BaseCommand
from django.test.client import Client

import sys
import os
from optparse import make_option

from staticblog.app_settings import STATICBLOG_POST_DIRECTORY, STATICBLOG_COMPILE_DIRECTORY

class Command(BaseCommand):
    help = "Compile blog posts from html to markdown, and upload images to S3 Defaults to processing only new blog posts"
    option_list = BaseCommand.option_list + (
        make_option('--all',
            action='store_true',
            dest='all',
            default=False,
            help='Get all blog posts, regardless of date'
        ),
        make_option('--name',
            action='store',
            dest='post_name',
            default=False,
            help='Get named blog post'
        ),
    )

    def handle(self, *args, **options):
        client = Client()
        outdir = STATICBLOG_COMPILE_DIRECTORY
        posts = []
        previews = []
        
        if options['all']:
            print 'Compiling all blog posts'
            posts = self._get_all_posts()
        elif options['post_name']:
            posts = self._get_named_posts(options['post_name'])
        else:
            print 'Compiling new blog posts'
            posts = self._get_all_posts(new = True)

        print '%d posts found' % len(posts)
        print '----------------------------'

        for post in posts:  
            print "Compiling " + post['md_name'] + " to " + post['html_name']
            path = '/preview/' + post['path']
            resp = client.get(path)
            if os.path.exists(outdir + post['path']) == False:
                try:
                    with open(outdir + post['path'], 'r') as f: 
                        pass
                except IOError as e:
                    os.mkdir(outdir + post['path'])

            with open(outdir + post['html_name'], 'w') as f:
                f.write(resp.content) 

        if len(posts) > 0:
            print '----------------------------'

        print 'Updating listings...'
        print '----------------------------'
        path = '/preview/'
        resp = client.get(path)

        with open(STATICBLOG_COMPILE_DIRECTORY + 'index.html', 'w') as f:
            f.write(resp.content) 
        
        print 'Done'


    def _get_all_posts(self, new = False):    
        posts = []
        for item in os.listdir(STATICBLOG_POST_DIRECTORY):
            post = self._create_post(item, new)
            if post:
                posts.append(post)
        return posts

    def _get_named_posts(self, post):
        post_list = post.split(',')
        posts = []
        for item in post_list:
            try:
                with open(STATICBLOG_POST_DIRECTORY + item, 'r') as f: 
                    post = self._create_post(item)
                    if post:
                        posts.append(post)
            except IOError as e:
                print >> sys.stderr, '\033[01;31m' + str(e) + '\033[0m'
        return posts

    def _create_post(self, item, new = False):
        outdir = STATICBLOG_POST_DIRECTORY
        compiled_post = {
            'md_name' : item,
            'html_name' : '',
            'path' : '',
            'html' : '',
        }
        if item.endswith('.post'):
            compiled_post['path'] = item.replace('.post', '')
            compiled_post['html_name'] = compiled_post['path'] + '/index.html'
            if new:
                try:
                    with open(STATICBLOG_COMPILE_DIRECTORY + compiled_post['html_name'], 'r') as f: 
                        return False
                except IOError as e:
                    return compiled_post
            else:
                return compiled_post   



from django.core.management.base import BaseCommand

from staticblog.app_settings import STATICBLOG_POST_DIRECTORY

import time
import json
import urllib2
import html2text

class Command(BaseCommand):
    help = "Import blog posts from tumblr"
    args = "<blog_url>"

    def handle(self, *args, **options):
        if len(args) != 1:
            print 'Incorrect arguments'
            return False
        url = args[0]
        if not 'http://' in url and not 'https://' in url:
            url = 'http://' + url
        url += '/api/read/json?debug=1'
        blog = urllib2.urlopen(url)
        blog = json.loads(blog.read())

        for post in blog['posts']:
            meta = ''
            if post['slug'] != '':
                filename = post['slug'] + '.post'
            else:
                filename = post['id'] + '.post'
            if post['type'] == 'regular':
                markdown = html2text.html2text(post['regular-body'])
                if post['regular-title'] == None:
                    post['regular-title'] = ' '
                meta = "Title: " + post['regular-title'] + "\n"
            elif post['type'] == 'audio':
                markdown = html2text.html2text(post['audio-caption'])
                markdown += "\n" + post['audio-player']
                meta = "Title: " + "\n"
            meta += "Summary: " + "\n"
            meta += "Author: " + "\n"
            meta += "Date: " + post['date-gmt'] + "\n"
            meta += "\n" 

            print "Importing %s" % filename

            with open(STATICBLOG_POST_DIRECTORY + filename, 'w') as f:
                f.write(meta)
                f.write(markdown.encode('UTF-8'))

            time.sleep(1)
                
        
        



from setuptools import setup, find_packages
import staticblog

print find_packages()

setup(
    name = 'django-staticblog',
    version = staticblog.__version__,
    packages = find_packages(),
    install_requires = ['Markdown >= 2.0', 'html2text >= 3.0'],
    author = 'Chris Grice',
    author_email = 'chris@chrisgrice.com',
    license = 'MIT',
    description = 'Markdown-based blog engine that compiles to static html pages',
    url='http://github.com/cgrice/django-staticblog/',
    classifiers = [
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    zip_safe = False,
    package_data = { 'staticblog' : ['templates/staticblog/*.html']},
    include_package_data = True,
)
